function oData = data( varargin )
% DATA model class constructor. 
% s = data(T,X,ModelName,Legend,Output,vk,Volume,Date,IntegratorType)
%==========================================================================
% T              => Time vector.
% X              => Vector of state vectors.
% ModelName      => Nominal initial condition vector
% Legend         => Vector of strings.Names of various species involved.
% Output         => Expression consisting of state variables to be used
%                   as output. Is is a string.
% vk             => Parameter vector.
% Volume         => Volume.
% Date           => Date and time of simulation.
% IntegratorType => type of integrator used eg. Gillespie,MATLAB,MEX etc
%==========================================================================
% All the parameters are required to create the objects. 
% All the parameters must appear in the given order.
% Objects can also be created by passing an object of type DATA to
% this function.

rData.ModelNameComment    = 'Model name             :';% comments for the 'display' function
rData.DateComment         = 'Date of Simulation     :';
rData.vkComment           = 'Parameters             :';
rData.VolumeComment       = 'Volume                 :';
rData.LegendComment       = 'Legend                 :';
rData.OutputComment       = 'Output                 :';
rData.IntegratorComment   = 'Integrator Type        :';


switch (nargin)
    
       
    case 1
%       If a single argument of class data, return it
        if (isa(varargin{1},'data'));
            oData = varargin{1};
        else
            error('Input argument is not an object of class data');
        end

        
    case 9
        
%       check for correct input arguments

        if (~isnumeric(varargin{1}))
            error('Incorrect first input argument. See HELP');
        elseif(~isnumeric(varargin{2}) &&  ~isstruct(varargin{2}))
            error('Incorrect second input argument. See HELP');
        elseif(~ischar(varargin{3}))
            error('Incorrect third input argument. See HELP');
        elseif(~isvector(varargin{4}))
            error('Incorrect fourth input argument. See HELP');
        elseif(~ischar(varargin{5}))
            error('Incorrect fifth input argument. See HELP');
        elseif(~isvector(varargin{6}))
            error('Incorrect sixth input argument. See HELP');
        elseif(~isnumeric(varargin{7}))
            error('Incorrect seventh input argument. See HELP');    
        elseif(~ischar(varargin{8}))
            error('Incorrect eighth input argument. See HELP');
        elseif(~ischar(varargin{9}))
            error('Incorrect eighth input argument. See HELP');
        end
%       Create object using specified values, keep default for rest

        rData.ModelName = varargin{3};% model_name
        rData.vk = varargin{6};% parameters
        rData.X = varargin{2};% state vectors  
        rData.Output = varargin{5};% output
        rData.Legend = varargin{4};% legend
        rData.T = varargin{1};% time vector
        rData.Date = varargin{8};% Date and time of simulation
        rData.Volume =(varargin{7});% volume
        rData.IntegratorType =(varargin{9}); % Integrator type
        oData = class(rData,'data');% return the class object
        
    otherwise
        error ('Invalid number of arguments. See HELP');
        
end
end
