function [display, handles] = analyzemultiMCPCA(allgoods, allsets)

% analyzemultiMCPCA
%
% [display, handles] = analyzemultiMCPCA(allgoods, allsets)
%
%   Plot the distribution of the multi_MC_alongPCA results
%   allsets optional
%
%   outputs = std deviation and eigendirection of the goods sets
%             handle to the plots
%
% Version 1.1, november 2009
% Marc Hafner, EPFL-IC-LANOS, Switzerland
% help and comments to marc.hafner@epfl.ch
%


%cleaning non-variating parameters

vark = var(allgoods{1});
kstoplot = sort((vark>1e-10).*[1:length(vark)],'descend');
kstoplot = sort(kstoplot(1:max((kstoplot~=0).*[1:length(kstoplot)])));

lk=length(kstoplot);

setstoplot = length(allgoods);
for i=1:setstoplot
    fallgoods{i} = allgoods{i}(:,kstoplot);
    if exist('allsets','var')
        fallsets{i} = allsets{i}(:,kstoplot);
    else
        fallsets{i} = mean(allgoods{i}(:,kstoplot));
    end
end

ksign = sign(mean(fallgoods{1}));


if ~exist('allsets','var')
    for i=1:setstoplot
        allsets{i}(1,:) = max(fallgoods{i});
        allsets{i}(2,:) = min(fallgoods{i});
    end
end

for i=1:setstoplot
    maxk(i,:) = max(fallsets{i});
    mink(i,:) = min(fallsets{i});
end

maxk = max(maxk).*(1+.1*ksign);
mink = min(mink).*(1-.1*ksign);


for majori = 1:setstoplot
    figure(majori)
    clf

    h = plotks(fallsets{majori},'.','color',[.5 .6 .5],'markersize',14);
    hold on
    h = plotks(fallgoods{majori},'.','color',[0 0 .8],'markersize',14);
    hold off
    standard_graph(h);
    
    legend(h.sub{end}.f,'tested parameter','viable parameter');

    for i=1:length(h.sub)
        
        axes(h.sub{i}.f)
        
        xlim([mink(2*i-1) maxk(2*i-1)])
        minxtick = floor(log10(ksign(2*i-1)*mink(2*i-1)));
        maxxtick = ceil(log10(ksign(2*i-1)*maxk(2*i-1)));
        xticks = [minxtick (minxtick+ksign(2*i-1)):ceil((maxxtick-minxtick)/4):(maxxtick-ksign(2*i-1)) maxxtick];
        if 1% ksign(2*i-1)>0   
            ksign(2*i-1)*10.^xticks;
            set(h.sub{i}.f,'XTick',ksign(2*i-1)*10.^xticks)
        else
            ksign(2*i-1)*10.^xticks(end:-1:1);
            set(h.sub{i}.f,'XTick',ksign(2*i-1)*10.^xticks(end:-1:1))
        end
        
        if i==length(h.sub) && mod(lk,2)==1
            jj = 2*i-2;
        else
            jj = 2*i;
        end
        ylim([mink(jj) maxk(jj)])
        minytick = floor(log10(ksign(jj)*mink(jj)));
        maxytick = ceil(log10(ksign(jj)*maxk(jj)));
        yticks = [minytick (minytick+ksign(jj)):ceil((maxytick-minytick)/4):maxytick-(ksign(jj)) maxytick];
        
        ksign(jj)*10.^yticks;
        set(h.sub{i}.f,'YTick',ksign(jj)*10.^yticks)
        eval(sprintf('xlabel(''k_{%i}'');',kstoplot(2*i-1)))
        if 2*i<=length(kstoplot)
            eval(sprintf('ylabel(''k_{%i}'');',kstoplot(2*i)));
        else
            eval(sprintf('ylabel(''k_{%i}'');',kstoplot(2*i-2)));
        end

        set(h.sub{i}.f,'XMinorTick','on', 'YMinorTick','on');

    end
    
    legend(h.sub{end}.f,'tested parameter','viable parameter');

    handles(majori) = h;
end



for i=1:setstoplot
    logks = log10(fallgoods{i}.*(ones(size(fallgoods{i},1),1)*ksign));
    [axis, coord, latent] = princomp(logks);
    display{i} = [sqrt(latent) ones(length(ksign),1)*i axis];
end