%
% Script for the parameter search with an iterative Monte-Carlo mehtod.
%
% Model should be renamed as oModel. PromptMode=0 for no interface
% Parameters for simulations are (used as defaults if already exist):
%   - 'simulationnumber' , number for autobackup/recovery/saved data
%   - 'kcheckfunction' , handle to function(oModel, vk [, mainX0])
%           returning a logical value. Name should be kcheck_modelname*.m
%           where modelname is the model used and * can be anything.
%   - 'mainX0' , initial conditions for kcheckfunction
%   - 'nbsets' , # of parameters tested per step
%   - 'nbhits' , # of maximal viable parameter per step
%   - 'maxmajor' , maximal # of step
%
% See also VOLUMEMC_ALONG_PCA.
%
% Version 1.1, november 2009
% Marc Hafner, EPFL-IC-LANOS, Switzerland
% help and comments to marc.hafner@epfl.ch
%

%% needed variables

functionname = 'multiMC_along_PCA';
eval(sprintf('i0name = ''%si0'';',functionname));
eval(sprintf('majori0name = ''%smajori0'';',functionname));

if exist('oModel','var')==0
    disp('Rename the model to be used as ''oModel'' ')
    break;
else
    disp(['Model used: ' oModel.tModelName])
end

global PromptMode
if isempty(PromptMode)==1
    displayPromptModes();
    PromptMode = input('Please set PromptMode, 0-1: ');
    if isempty(PromptMode)
        PromptMode=1;
    end
end

global DisplayMode
if isempty(DisplayMode)==1
    displayDisplayModes();
    DisplayMode = input('Please set DisplayMode, 0-5: ');
    if isempty(DisplayMode)
        DisplayMode=3;
    end
end

SelectWorkingPath

%% checking for old simulations


skip=0;
if exist(i0name,'var') || exist(majori0name,'var')
    if PromptMode; reply = input('On going simulation, do you want to continue it y/n [n]? ', 's');
    else reply='n';end;
    if isempty(reply)==1 || reply=='N' || reply=='n'
        clear(i0name, majori0name);
    else
        skip=1;
    end
end

if ~exist([bioSimworkingpath filesep 'backup' filesep],'dir')
    mkdir([bioSimworkingpath filesep 'backup' filesep])
end

if skip==0

    if ~exist('simulationnumber','var') || ~isfinite(simulationnumber);simulationnumber = 1;end;      % simulation number
    if PromptMode; reply = input(['Simulation number ' num2str(simulationnumber) ', do you agree y/1-.../n [y]? '], 's');
    else reply='y';end;

    if ~isempty(reply) && ~isempty(str2double(reply))
        if reply(1)~='y' && reply(1)~='Y' 
            simulationnumber = floor(str2double(reply));
        end
    elseif isempty(reply)==0 && reply(1)~='Y' && reply(1)~='y'
        simulationnumber = floor(input('Choose simulation number: '));
    end
    
    tempname = sprintf('%s%sbackup%s%s_data_%s_temp%i.mat',bioSimworkingpath,filesep,filesep,functionname,oModel.tModelName,simulationnumber);
    name = sprintf('%s%sbackup%s%s_data_%s_%i.mat',bioSimworkingpath,filesep,filesep,functionname,oModel.tModelName,simulationnumber);
    
    if exist(name,'file') && exist(tempname,'file')
        disp(['found old simulation: ' name ])
        disp(['found old simulation: ' tempname ])
        if PromptMode; reply = input('Do you want to load these data y/n [y]? ', 's');
        else reply='y';end;
        if isempty(reply)==1 || reply=='Y' || reply=='y'
            load(name)
            load(tempname)
            disp([name ' loaded'])
            disp([tempname ' loaded'])
        end
    elseif exist(name,'file')
        disp(['found old simulation: ' name ])
        if PromptMode; reply = input('Do you want to load these data y/n [y]? ', 's');
        else reply='y';end;
        if isempty(reply)==1 || reply=='Y' || reply=='y'
            load(name)
            disp([name ' loaded'])
        end
    elseif exist(tempname,'file')
        disp(['found old simulation: ' tempname])
        if PromptMode; reply = input('Do you want to load these data y/n [y]? ', 's');
        else reply='y';end;
        if isempty(reply)==1 || reply=='Y' || reply=='y'
            load(tempname)
            disp([tempname ' loaded'])
        end
    end
end


%% parameter conditions
if skip==0 && (~exist('kcheckfunction','var') || (nargin(kcheckfunction)~=2 && nargin(kcheckfunction)~=3) )

    ckfunctions = dir(['kcheck_' oModel.tModelName '*.m']);

    if ~isempty(ckfunctions)
        if PromptMode;
            disp('Aviable function for conditions on parameter:');
            for i=1:length(ckfunctions)
                disp(['    ' num2str(i) '    ' ckfunctions(i).name]);
            end
            reply = input(['Choose which function do you want to use 1-' num2str(length(ckfunctions)) ' [1]? '], 's');
        else reply='1';
        end


        if isempty(reply), reply='1'; end;
        ckfunctions = ckfunctions(str2num(reply)).name;
        pointposition = max( (ckfunctions=='.') .* [1:length(ckfunctions)] );
        eval(sprintf('kcheckfunction = @%s;', ckfunctions(1:pointposition-1) ));

        if nargin(kcheckfunction)~=2 && nargin(kcheckfunction)~=3
            error(['Inputs for function with conditions on parameter found should be (oModel, vk [, vX0])'])
        end
        if nargout(kcheckfunction)~=3
            error(['Inputs for function with conditions on parameter found should be (oModel, vk [, vX0])' ...
                '; outputs should be [test, X0, conditions]'])
        end
            
    else
        file = fopen(['kcheck_' oModel.tModelName '0.m'],'w');
        fprintf(file, ['function [test, X0, conditions] = kcheck_%s0 (oModel, vk, vX0)\n\n' ...
            '%% Enter the condition for vk (test = true or false)\n' ...
            '%% Enter an output ( X0 = state vector)\n' ...
            '%% Enter a condition vector (conditions = display output)\n\n' ], oModel.tModelName);
        fclose(file);
        open(['kcheck_' oModel.tModelName '0.m']);
        error(['No function with conditions on parameter found. Create a m file kcheck_' ...
            oModel.tModelName '*.m in the current folder. Inputs should be (oModel, vk [, vX0])' ...
            '; outputs should be [test, X0, conditions]'])
    end
end

%% initial conditions
if skip==0 && ~exist('mainX0','var') && nargin(kcheckfunction)>2
    disp([func2str(kcheckfunction) ' required initial conditions'])
    if PromptMode
        disp('No initial conditions specified')
        disp('Model default initial conditions are:')
        disp(oModel.vX0)
        reply = input('Do you want to use the model default initial conditions [y]/n?','s');
    else reply='y';
    end
    
    if ~isempty(reply) && (reply=='N' || reply=='n')
        error('Specify the initial conditions in vector ''mainX0''');
    else
        mainX0 = oModel.vX0;
    end
end


    
    
    

%% define the parameters of the simulation

disp('Used settings: ')

if ~exist('nbsets','var');nbsets = 5e4;end;       % nbre of tested set per step
disp(['   nbre of tested sets per step (nbsets): ' num2str(nbsets)])

if ~exist('nbhits','var');nbhits = 1e3;end;      % nbre of max goodks per step
disp(['   max nbre of good steps per step (nbhits): ' num2str(nbhits)])

if ~exist('maxmajor','var');maxmajor = 10;end;       % nbre of major loops
disp(['   max nbre of major loops (maxmajor): ' num2str(maxmajor)])

variancetimemax = 2.5;   % distribution for random sampling
variancetimemin = 1.5;
type = 'gaus';

disp(['Using conditions function: ' func2str(kcheckfunction)])

if nargin(kcheckfunction)>2
    disp('Using initial conditions')
    disp(mainX0)
end

disp(['Simulation number : ' num2str(simulationnumber)])

if PromptMode; reply = input('Do you want to perform the simulation with these settings y/n [y]? ', 's');
else reply='y';end;
if isempty(reply)==0 && (reply=='N' || reply=='n')
    break
end
disp([' '])


%% initialization part
time=clock;
rand('twister',1e7*simulationnumber+time(6)+100*time(5)+1e4*time(4)+1e6*time(3));
randn('seed',1e7*simulationnumber+time(6)+100*time(5)+1e4*time(4)+1e6*time(3));

eval(sprintf('logbook = fopen(''logbook_%s_%s_%i.txt'',''a'');',functionname,oModel.tModelName,simulationnumber));

fprintf(logbook,'\n ---- Day (MMDD) : %i/%i Time : %i:%i ----\n\n',time(2),time(3), time(4), time(5));

fprintf(logbook,'%s\n',['   nbre of tested sets per step: ' num2str(nbsets)]);
fprintf(logbook,'%s\n',['   max nbre of good steps per step: ' num2str(nbhits)]);
fprintf(logbook,'%s\n',['   max nbre of major loops: ' num2str(maxmajor)]);
fprintf(logbook,'using conditions function on the parameters: %s\n',func2str(kcheckfunction));
if nargin(kcheckfunction)>2
    fprintf(logbook,'Initial conditions: ');
    for i=1:length(mainX0)
        fprintf(logbook,'%.3e ',mainX0(i));
    end
    fprintf(logbook,'\n');
end


lk = oModel.slk - oModel.sconstrained;
lX = oModel.slX;
k0 = oModel.vk;
cc = k0(end-oModel.sconstrained+1:end);
k0 = k0(1:end-oModel.sconstrained);
ksign=sign(k0);

if exist(i0name,'var')==0 && exist(majori0name,'var')==0
    disp('main initialization');
    axis = eye(lk,lk);
    vari = .1*ones(lk,1);
    lastlogmean = log10(ksign.*k0);
    lastgoods = k0;
    eval(sprintf('%s=1;',majori0name));
    clear(i0name)
end

%% major loop

eval(sprintf('internmajori0 = %s;',majori0name));

for majori = internmajori0:maxmajor
    
    eval(sprintf('%s = majori;',majori0name));

    wwait = waitbar(0,['multiMC_along_PCA major loop #' num2str(majori)]);
    
    if exist('vari')==0
        if exist('variance')==0
            vari = .1*ones(lk,1);
        else
            vari = variance{end};
        end
    end

    time=clock;
    fprintf(logbook,'\n ---- Day (MMDD) : %i/%i Time : %i:%i ----\n\n',time(2),time(3), time(4), time(5));
    fprintf(logbook,'Starting major loop %i; ',majori);

    fprintf ('Starting major loop %i (%i sets) : ', majori, nbsets);


    if exist(i0name,'var')==0
        variancetime = variancetimemin + (variancetimemax-variancetimemin)*max([1-majori/5 0]);
        % generate the parameters sets

        if strcmp(type,'unif')
            % sets are equvalently distribute along the major axis in the +/-
            % variancetime * variance interval
            positions = (2*rand(nbsets,lk) -1) .* ( variancetime*ones(nbsets,1)*vari');

        elseif strcmp(type,'gaus')
            % sets are Gaussianly distribute along the major axis in the +/-
            % variancetime * variance interval
            positions = (randn(nbsets,lk)) .* ( variancetime*ones(nbsets,1)*vari');

        end

        sets = (axis*positions')';
        sets = sets + ones(nbsets,1)*lastlogmean(1:lk);
        sets = (10.^sets).*(ones(nbsets,1)*ksign(1:lk));
        %
        % coord = (axis*coord')';
        % coord = coord + ones(length(goodks),1)*mean(logks);
        % coord = 10.^coord;



        % test the new sets for viable results

        clear newgoods
        interni0=1;
    else
        eval(sprintf('interni0 = %s;',i0name));
    end

    for i=interni0:nbsets
        if mod(i,100)==0
            fprintf('Major %i ; set %i\n ',majori,i); 
            waitbar(i/nbsets);
        end
        if DisplayMode>2; fprintf('Major %i ; set %i\n ',majori,i); end
        
        if nargin(kcheckfunction)>2
            [test junkX0 conditions] = kcheckfunction(oModel, [sets(i,:) cc], mainX0);
        else
            [test junkX0 conditions] = kcheckfunction(oModel, [sets(i,:) cc]);
        end
        
        
        if test
            if exist('newgoods')==0
                newgoods = sets(i,:);
            else
                newgoods(end+1,:) = sets(i,:);
            end

            if length(newgoods)>=nbhits
                sets(i+1:end,:)=[];
                break
            end

        else
            if DisplayMode>4
                fprintf('Major %i ; set %i refused due to kcheckfunction ',majori,i);
                disp(conditions)
            end
        end

        if mod(i,2500)==400
            eval(sprintf('%s = i+1;',i0name));
            fprintf('Saving at set %i ',i);
            fprintf(logbook,'Saving at set: %i;\n', i);
            if exist('newgoods')
                save(tempname, majori0name, 'newgoods', i0name, 'mainX0', 'sets', 'kcheckfunction')
            else
                save(tempname, majori0name, i0name, 'mainX0', 'sets', 'kcheckfunction')
            end

        end

    end
    close(wwait)
    %     eval(sprintf('allgoods%i = newgoods;',majori));
    allgoods{majori} = [newgoods ones(size(newgoods,1),1)*cc];
    allsets{majori} = [sets ones(size(sets,1),1)*cc];
    clear('sets',i0name)

    vark = var(newgoods);
    vark = sort((vark~=0).*[1:length(vark)],'descend');
    vark = sort(vark(1:max((vark~=0).*[1:length(vark)])));

    logks = log10ks(newgoods(:,vark));
    
    [axis2, coord, latent] = princomp(logks);
    axis = zeros(lk,lk);
    axis(vark,vark) = axis2;
    variance{majori} = zeros(lk,1);
    variance{majori}(vark) = sqrt(latent);
    vari = variance{majori};
    lastlogmean = -Inf*ones(1,lk);
    lastlogmean(vark) = mean(logks);


    eval(sprintf('%s = majori+1;',majori0name));
    save(name, 'allsets', 'allgoods', 'variance', majori0name, 'kcheckfunction')
    delete(tempname)
    fprintf(' ; loop over, data saved \n');
    fprintf(logbook,'\nloop over, data saved \n');
    fprintf(logbook,'Variances:');
    for i=1:lk
        fprintf(logbook,' %.3e', variance{majori}(i));
    end

    if majori>1
        if max(abs((variance{majori}./variance{majori-1}) -1))< .1
            fprintf(logbook,'\nConvergence of variances (<0.1) :\n')
            for i=1:lk
                fprintf(logbook,' %.3e', variance{majori}(i)./variance{majori-1}(i)-1 );
                fprintf(' %.3e', variance{majori}(i)./variance{majori-1}(i)-1 );
            end
            fprintf(logbook,'\n Major loop over, end gathering data\n\n\n');
            fprintf('\n Major loop over, end gathering data\n\n\n');
            break
        else
            fprintf(logbook,'\nNo convergence of variances (<0.1) :\n')
            fprintf('\nNo convergence of variances (<0.1) :\n')
            for i=1:lk
                fprintf(logbook,' %.3e', variance{majori}(i)./variance{majori-1}(i)-1 );
                fprintf(' %.3e', variance{majori}(i)./variance{majori-1}(i)-1 );
            end
        end
    end

    fprintf(logbook,'\n next major loop\n\n\n');
end

    %%
eval(sprintf('save ./%s_results_%s_%i.mat allsets allgoods variance kcheckfunction mainX0',functionname,oModel.tModelName,simulationnumber));

clear(majori0name, i0name); 
time=clock;
fprintf(logbook,'\n ---- Day (MMDD) : %i/%i Time : %i:%i ----\n\n',time(2),time(3), time(4), time(5));
fclose(logbook)



%% plotting both distributions

display = analyzemultiMCPCA(allgoods, allsets);

if DisplayMode>2
    for i=1:length(display)
        disp(display{i})
    end
end
