function o = optionget(options,name,default,flag)
%OPTIONGET Get ODE OPTIONS parameters.
%   VAL = OPTIONGET(OPTIONS,'NAME') extracts the value of the named property
%   from integrator options structure OPTIONS, returning an empty matrix if
%   the property value is not specified in OPTIONS. 
% undocumented usage for fast access with no error checking
if (nargin == 4) && isequal(flag,'fast')
   o = getknownfield(options,name,default);
   return
end

if nargin < 2
  error('MATLAB:optionget:NotEnoughInputs','Not enough input arguments.');
end
if nargin < 3
  default = [];
end

if ~isempty(options) && ~isa(options,'struct')
  error('MATLAB:optionget:Arg1NotOPTIONSETstruct',...
        'First argument must be an options structure created with OPTIONSET.');
end

if isempty(options)
  o = default;
  return;
end

Names = [
    'RelTime       '
    'AbsPre        '
    'Display       '
    'PlotVar       '
    'Style         '
    'Color         '
    'nbunits       '
    ];
names = lower(Names);

lowName = lower(name);
j = strmatch(lowName,names);
if isempty(j)               % if no matches
  error('MATLAB:optionget:InvalidPropName',...
        ['Unrecognized property name ''%s''.  ' ...
         'See OPTIONSET for possibilities.'], name);
elseif length(j) > 1            % if more than one match
  % Check for any exact matches (in case any names are subsets of others)
  k = strmatch(lowName,names,'exact');
  if length(k) == 1
    j = k;
  else
    msg = sprintf('Ambiguous property name ''%s'' ', name);
    msg = [msg '(' deblank(Names(j(1),:))];
    for k = j(2:length(j))'
      msg = [msg ', ' deblank(Names(k,:))];
    end
    msg = sprintf('%s).', msg);
    error('MATLAB:optionget:AmbiguousPropName', msg);
  end
end

if any(strcmp(fieldnames(options),deblank(Names(j,:))))
  o = options.(deblank(Names(j,:)));
  if isempty(o)
    o = default;
  end
else
  o = default;
end

% --------------------------------------------------------------------------
function v = getknownfield(s, f, d)
%GETKNOWNFIELD  Get field f from struct s, or else yield default d.

if isfield(s,f)   % s could be empty.
  v = subsref(s, struct('type','.','subs',f));
  if isempty(v)
    v = d;
  end
else
  v = d;
end

% code based on the implementation done in
% Matlab\toolbox\matlab\funfun\odeset.m