function oData=simulation_variational(oModel,tspan,varargin)

%SIMULATION_VARIATIONAL Deterministic variational simulation.
%   SIMULATION_VARIATIONAL (oModel,tspan,vk,vX0)
%
%   It returns the data object oData.
%   vk and vX0 are optional,if vX0 is given as input it must be preceded by
%   vk, whereas vk can be given alone.
%
%   eg. simulation_variational(a1,[50 2000],vk,vX0)
%
% See also DATA.
%
% Version 1, october 2008
% Marc Hafner, EPFL-IC-LANOS, Switzerland
% help and comments to marc.hafner@epfl.ch
%



k=oModel.vk;
X0=oModel.vX0;
lX = length(X0);
tnargin=nargin;
options=odeset();
eval(sprintf('integrator = @%sVariationalIntegrator;',oModel.tModelName));

if tnargin < 2
    error ('insufficient  number of input agruments');
    
elseif tnargin == 2
    %     default case with only 2 input arguments.
    
elseif tnargin == 3 && isvector(varargin{1})
    %     if 3rd argument is vector k
    if length(varargin{1})~=length(oModel.vk)
        error('Wrong input argument. Check length of vector vk');
    else
        k=varargin{1};
    end

elseif tnargin == 4 && isvector(varargin{2})
    %     if 4th argument is vector X0, clearly 3rd is vk
    if length(varargin{1})~=length(oModel.vk)
        error('Wrong input argument. Check length of vector vk');
    else
        k=varargin{1};
    end
    if length(varargin{2})~=lX
        error('Wrong input argument. Check length of vector vX0');
    else
        X0=varargin{2};
    end
elseif tnargin == 5 && isvector(varargin{1}) && isvector(varargin{2})
    if length(varargin{1})~=length(oModel.vk)
        error('Wrong input argument. Check length of vector vk');
    else
        k=varargin{1};
    end
    if length(varargin{2})~=lX
        error('Wrong input argument. Check length of vector vX0');
    else
        X0=varargin{2};
    end
    options = varargin{3};
else
    error ('Wrong input arguments.See HELP');

end

[T,X]=integrator(tspan,k,X0,options);

XdX.vX = X(:,1:lX);
XdX.dX = zeros(length(T),lX,lX);
for i=1:lX
    for j=1:lX
        XdX.dX(:,i,j) = X(:,lX*i+j);
    end
end

date = datestr(now);
integratortype='Variational Matlab';
eval('oData = data(T,XdX,oModel.tModelName,oModel.vlegend,oModel.tOutputExpression,oModel.vk,0,date,integratortype);');

end





