function [T,X] = GoodwinODEIntegrator(vTimeRange, k, vX0, options)

if nargin<3 || length(k)~=10 || length(vX0)~=3 || length(vTimeRange)>3
	error('Incorrect arguments')
end

lX=3;

n=4;

function dX = F(t,X)
	dX = X;

	 % ... write ODE for all variables 

	dX(1) =  k(1)*(k(2)^n)/(k(2)^n+X(3)^n) -  k(3)*X(1)/(k(4)+X(1));
    dX(2) =  k(5)*X(1) -  k(6)*X(2)/(k(7)+X(2));
    dX(3) =  k(8)*X(2) -  k(9)*X(3)/(k(10)+X(3));
end

function df = dFdX(t,X)
	df = zeros(lX,lX);

	 % ... write ODE derivates for all variables 

	df(1,1) = -k(3)*k(4)/((k(4)+X(1))^2); % to be filled by hand !
	df(1,3) = -n*k(1)*(k(2)^n)*(X(3)^(n-1))/((k(2)^n+X(3)^n)^2) ; % to be filled by hand !

	df(2,1) = k(5) ; % to be filled by hand !
	df(2,2) = -k(6)*k(7)/((k(7)+X(2))^2) ; % to be filled by hand !

	df(3,2) = k(8) ; % to be filled by hand !
	df(3,3) = -k(9)*k(10)/((k(10)+X(3))^2) ; % to be filled by hand !

end

% end of editable part of this file. Unless you want to optimize, leave
% the rest of this file original.


if length(vTimeRange)==1
    X=F(vTimeRange,vX0);
    T=vTimeRange;
    return
else
    if length(vTimeRange)==3
        vTimePoints = [0:vTimeRange(2):vTimeRange(3)];
    else
        vTimePoints = [0 vTimeRange(2)];
    end
    if exist('options','var')
        options = odeset(options,'Jacobian',@dFdX);
    else
        options = odeset('Jacobian',@dFdX);
    end
    [T,X]=ode45(@F,vTimePoints,vX0,options);
end

irelax = max([1; [1:length(T)]'.*(T<vTimeRange(1))]);
T=T(irelax:end);
X=X(irelax:end,:);

end
