function [T,X] = GoodwinVariationalIntegrator(vTimeRange, k, vX0, options)

if nargin<3 || length(k)~=10 || length(vX0)~=3 || length(vTimeRange)>3
	error('Incorrect arguments')
end

lX=3;

n=4;

function dX = F(t,X)
	dX = X;

	 % ... write ODE for all variables 

	dX(1) =  k(1)*(k(2)^n)/(k(2)^n+X(3)^n) -  k(3)*X(1)/(k(4)+X(1));
    dX(2) =  k(5)*X(1) -  k(6)*X(2)/(k(7)+X(2));
    dX(3) =  k(8)*X(2) -  k(9)*X(3)/(k(10)+X(3));
end

function df = dFdX(t,X)
	df = zeros(lX,lX);

	 % ... write ODE derivates for all variables 

	df(1,1) = -k(3)*k(4)/((k(4)+X(1))^2); % to be filled by hand !
	df(1,3) = -n*k(1)*(k(2)^n)*(X(3)^(n-1))/((k(2)^n+X(3)^n)^2) ; % to be filled by hand !

	df(2,1) = k(5) ; % to be filled by hand !
	df(2,2) = -k(6)*k(7)/((k(7)+X(2))^2) ; % to be filled by hand !

	df(3,2) = k(8) ; % to be filled by hand !
	df(3,3) = -k(9)*k(10)/((k(10)+X(3))^2) ; % to be filled by hand !

end

% end of editable part of this file. Unless you want to optimize, leave
% the rest of this file original.

    function dX = FdF(t,X)
        dX = zeros(lX+lX*lX,1);

        dX(1:lX) = F(t,X(1:lX));
        df = dFdX(t,X(1:lX));

        for i=1:lX
            for j=1:lX
                for kk=1:lX
                    dX(lX*i+j) = dX(lX*i+j) + df(i,kk)*X(lX*kk +j);
                end
            end
        end
    end


for ii=1:lX
    for jj=1:lX
        vX0(lX*ii+jj) = ii==jj;
    end
end

if length(vTimeRange)==1
    X=FDF(vTimeRange,vX0);
    T=vTimeRange;
    return
else
    if length(vTimeRange)==3
        vTimeRange = [vTimeRange(1):vTimeRange(2):vTimeRange(3)];
    end
    if exist('options','var')
        [T,X]=ode45(@FdF,vTimeRange,vX0,options);
    else
        [T,X]=ode45(@FdF,vTimeRange,vX0);
    end
end

irelax = max([1; [1:length(T)]'.*(T<vTimeRange(1))]);
T=T(irelax:end);
X=X(irelax:end,:);

end
